@@ -42,17 +42,21 @@ getRemoteDocument function when this occurs.
 
 It may also be the case that actions performed on various HTML elements will
 cause the page to change in the browser. For example, if the click method is
-invoked on an HTML input element. After such actions, it is recommended that the
-special setReady method is used on the part object before attempting to get the
-changed document:
+invoked on an HTML input element. For such actions, it is recommended that the
+special syncObject method is used on the part object before to obtain a version
+of the original node or object before performing the action on that node or
+object:
 
-  input_element.click()
-  part.setReady(0)
+  sync_input_element = part.syncObject(input_element)
+  sync_input_element.click()
   doc = qtxmldom.getRemoteDocument(part)
 
-The above mechanism is not perfect and may be replaced by something that better
-ensures the presence of a newly loaded document before any attempt may be made
-to retrieve it.
+The above mechanism should ensure that after the click method's action is
+performed, the program will wait for a new document to be loaded before being
+allowed to interact with the browser again.
+
+Note that the part's openURL method is already synchronised and the part itself
+does not need to be wrapped in the above fashion.
 """
 
 from kdecore import KApplication
@@ -186,6 +190,9 @@ class AutomationRoot:
         return str(url)
 
     def openURL(self, url):
+        return self.syncObject()._openURL(url)
+
+    def _openURL(self, url):
         self.get_remote_object().openURL(url)
 
     def closeURL(self):
@@ -199,9 +206,15 @@ class AutomationRoot:
         status, ready = self.get_remote_object().isReady()
         return ready
 
+    # NOTE: No longer useful now that getGeneration is in use.
+
     def setReady(self, ready):
         self.get_remote_object().setReady(ready)
 
+    def getGeneration(self):
+        status, generation = self.get_remote_object().getGeneration()
+        return generation
+
     def getDocument(self):
         remote_object = self.get_remote_object()
         status, result = remote_object.getDocument()
@@ -215,8 +228,50 @@ class AutomationRoot:
 
     # Utility methods.
 
+    def syncObject(self, obj=None):
+        if obj is None:
+            obj = self
+        return AutomationSyncObject(obj, self)
+
+    def waitForGeneration(self, generation, delay=1):
+        while not self.isReady() or self.getGeneration() != generation:
+            time.sleep(delay)
+
     def wait(self, delay=1):
         while not self.isReady():
             time.sleep(delay)
 
+# Utility classes.
+
+class AutomationSyncObject:
+
+    """
+    An object which wraps another object, ensuring that any method invoked on
+    that object is synchronised with the remote part.
+    """
+
+    def __init__(self, obj, part):
+        self.obj = obj
+        self.part = part
+
+    def __getattr__(self, name):
+        return AutomationSyncMethod(self.obj, self.part, name)
+
+class AutomationSyncMethod:
+
+    """
+    An object which permits the calling of a method and the subsequent
+    synchronisation with the remote part.
+    """
+
+    def __init__(self, obj, part, name):
+        self.obj = obj
+        self.part = part
+        self.name = name
+
+    def __call__(self, *args, **kw):
+        generation = self.part.getGeneration()
+        result = getattr(self.obj, self.name)(*args, **kw)
+        self.part.waitForGeneration(generation + 1)
+
 # vim: tabstop=4 expandtab shiftwidth=4
